@@ -1,6 +1,6 @@
 // This module implements the portability layer for the Qt port of Scintilla.
 //
-// Copyright (c) 2008 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -16,18 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
-// 
-// This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-// INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-// granted herein.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -50,27 +40,33 @@
 
 #include <qpointarray.h>
 
+#include "SciNamespace.h"
+
 #include "Platform.h"
 #include "XPM.h"
 
 #include "Qsci/qsciscintillabase.h"
 #include "SciClasses.h"
 
+#include "FontQuality.h"
+
+
+QSCI_BEGIN_SCI_NAMESPACE
 
 // Type convertors.
-static QFont *PFont(FontID id)
+static QFont *PFont(FontID fid)
 {
-    return reinterpret_cast<QFont *>(id);
+    return reinterpret_cast<QFont *>(fid);
 }
 
-static QWidget *PWindow(WindowID id)
+static QWidget *PWindow(WindowID wid)
 {
-    return reinterpret_cast<QWidget *>(id);
+    return reinterpret_cast<QWidget *>(wid);
 }
 
-static SciPopup *PMenu(MenuID id)
+static QsciSciPopup *PMenu(MenuID mid)
 {
-    return reinterpret_cast<SciPopup *>(id);
+    return reinterpret_cast<QsciSciPopup *>(mid);
 }
 
 
@@ -82,37 +78,8 @@ Point Point::FromLong(long lpoint)
 }
 
 
-// Colour palette management.  The Qt interface to colours means this class
-// doesn't have to do anything.
-Palette::Palette()
-{
-    used = 0;
-    allowRealization = false;
-}
-
-Palette::~Palette()
-{
-    Release();
-}
-
-void Palette::Release()
-{
-    used = 0;
-}
-
-void Palette::WantFind(ColourPair &cp, bool want)
-{
-    if (!want)
-        cp.allocated.Set(cp.desired.AsLong());
-}
-
-void Palette::Allocate(Window &)
-{
-}
-
-
 // Font management.
-Font::Font() : id(0)
+Font::Font() : fid(0)
 {
 }
 
@@ -120,33 +87,80 @@ Font::~Font()
 {
 }
 
-void Font::Create(const char *faceName, int, int size, bool bold, bool italic,
-        bool)
+void Font::Create(const FontParameters &fp)
 {
     Release();
 
     QFont *f = new QFont();
 
-    // If name of the font begins with a '-', assume, that it is an XLFD.
-    if (faceName[0] == '-')
-        f->setRawName(faceName);
-    else
+    QFont::StyleStrategy strategy;
+
+    switch (fp.extraFontFlag & SC_EFF_QUALITY_MASK)
     {
-        f->setFamily(faceName);
-        f->setPointSize(size);
-        f->setBold(bold);
-        f->setItalic(italic);
+    case SC_EFF_QUALITY_NON_ANTIALIASED:
+        strategy = QFont::NoAntialias;
+        break;
+
+    case SC_EFF_QUALITY_ANTIALIASED:
+        strategy = QFont::PreferAntialias;
+        break;
+
+    default:
+        strategy = QFont::PreferDefault;
     }
 
-    id = f;
+#if defined(Q_OS_MAC)
+#if QT_VERSION >= 0x040700
+    strategy = static_cast<QFont::StyleStrategy>(strategy | QFont::ForceIntegerMetrics);
+#else
+#warning "Correct handling of QFont metrics requires Qt v4.7.0 or later"
+#endif
+#endif
+
+    f->setStyleStrategy(strategy);
+
+    // If name of the font begins with a '-', assume, that it is an XLFD.
+    if (fp.faceName[0] == '-')
+    {
+        f->setRawName(fp.faceName);
+    }
+    else
+    {
+        f->setFamily(fp.faceName);
+        f->setPointSize(fp.size);
+
+        // See if the Qt weight has been passed via the back door.   Otherwise
+        // map Scintilla weights to Qt weights ensuring that the SC_WEIGHT_*
+        // values get mapped to the correct QFont::Weight values.
+        int qt_weight;
+
+        if (fp.weight < 0)
+            qt_weight = -fp.weight;
+        else if (fp.weight <= 200)
+            qt_weight = QFont::Light;
+        else if (fp.weight <= QsciScintillaBase::SC_WEIGHT_NORMAL)
+            qt_weight = QFont::Normal;
+        else if (fp.weight <= 600)
+            qt_weight = QFont::DemiBold;
+        else if (fp.weight <= 850)
+            qt_weight = QFont::Bold;
+        else
+            qt_weight = QFont::Black;
+
+        f->setWeight(qt_weight);
+
+        f->setItalic(fp.italic);
+    }
+
+    fid = f;
 }
 
 void Font::Release()
 {
-    if (id)
+    if (fid)
     {
-        delete PFont(id);
-        id = 0;
+        delete PFont(fid);
+        fid = 0;
     }
 }
 
@@ -161,46 +175,47 @@ public:
     void Init(WindowID wid);
     void Init(SurfaceID sid, WindowID);
     void Init(QPainter *p);
-    void InitPixMap(int width, int height, Surface *, WindowID);
+    void InitPixMap(int width, int height, Surface *, WindowID wid);
 
     void Release();
     bool Initialised() {return painter;}
-    void PenColour(ColourAllocated fore);
-    int LogPixelsY() {return 72;}
+    void PenColour(ColourDesired fore);
+    int LogPixelsY() {return pd->logicalDpiY();}
     int DeviceHeightFont(int points) {return points;}
     void MoveTo(int x_,int y_);
     void LineTo(int x_,int y_);
-    void Polygon(Point *pts, int npts, ColourAllocated fore,
-            ColourAllocated back);
-    void RectangleDraw(PRectangle rc, ColourAllocated fore,
-            ColourAllocated back);
-    void FillRectangle(PRectangle rc, ColourAllocated back);
+    void Polygon(Point *pts, int npts, ColourDesired fore,
+            ColourDesired back);
+    void RectangleDraw(PRectangle rc, ColourDesired fore,
+            ColourDesired back);
+    void FillRectangle(PRectangle rc, ColourDesired back);
     void FillRectangle(PRectangle rc, Surface &surfacePattern);
-    void RoundedRectangle(PRectangle rc, ColourAllocated fore,
-            ColourAllocated back);
-    void AlphaRectangle(PRectangle rc, int cornerSize, ColourAllocated fill,
-            int alphaFill, ColourAllocated outline, int alphaOutline,
+    void RoundedRectangle(PRectangle rc, ColourDesired fore,
+            ColourDesired back);
+    void AlphaRectangle(PRectangle rc, int cornerSize, ColourDesired fill,
+            int alphaFill, ColourDesired outline, int alphaOutline,
             int flags);
-    void Ellipse(PRectangle rc, ColourAllocated fore, ColourAllocated back);
+    void DrawRGBAImage(PRectangle rc, int width, int height, const unsigned char *pixelsImage);
+    void Ellipse(PRectangle rc, ColourDesired fore, ColourDesired back);
     void Copy(PRectangle rc, Point from, Surface &surfaceSource);
 
-    void DrawTextNoClip(PRectangle rc, Font &font_, int ybase, const char *s,
-            int len, ColourAllocated fore, ColourAllocated back);
-    void DrawTextClipped(PRectangle rc, Font &font_, int ybase, const char *s,
-            int len, ColourAllocated fore, ColourAllocated back);
-    void DrawTextTransparent(PRectangle rc, Font &font_, int ybase,
-            const char *s, int len, ColourAllocated fore);
-    void MeasureWidths(Font &font_, const char *s, int len, int *positions);
-    int WidthText(Font &font_, const char *s, int len);
-    int WidthChar(Font &font_, char ch);
-    int Ascent(Font &font_);
-    int Descent(Font &font_);
-    int InternalLeading(Font &font_) {return 0;}
-    int ExternalLeading(Font &font_);
-    int Height(Font &font_);
-    int AverageCharWidth(Font &font_) {return WidthChar(font_, 'n');}
+    void DrawTextNoClip(PRectangle rc, Font &font_, XYPOSITION ybase,
+            const char *s, int len, ColourDesired fore, ColourDesired back);
+    void DrawTextClipped(PRectangle rc, Font &font_, XYPOSITION ybase,
+            const char *s, int len, ColourDesired fore, ColourDesired back);
+    void DrawTextTransparent(PRectangle rc, Font &font_, XYPOSITION ybase,
+            const char *s, int len, ColourDesired fore);
+    void MeasureWidths(Font &font_, const char *s, int len,
+            XYPOSITION *positions);
+    XYPOSITION WidthText(Font &font_, const char *s, int len);
+    XYPOSITION WidthChar(Font &font_, char ch);
+    XYPOSITION Ascent(Font &font_);
+    XYPOSITION Descent(Font &font_);
+    XYPOSITION InternalLeading(Font &font_) {return 0;}
+    XYPOSITION ExternalLeading(Font &font_);
+    XYPOSITION Height(Font &font_);
+    XYPOSITION AverageCharWidth(Font &font_);
 
-    int SetPalette(Palette *, bool) {return 0;}
     void SetClip(PRectangle rc);
     void FlushCachedState();
 
@@ -210,11 +225,15 @@ public:
     void DrawXPM(PRectangle rc, const XPM *xpm);
 
 private:
+    void drawRect(const PRectangle &rc);
+    void drawText(const PRectangle &rc, Font &font_, XYPOSITION ybase,
+            const char *s, int len, ColourDesired fore);
+    static QFont convertQFont(Font &font);
     QFontMetrics metrics(Font &font_);
     QString convertText(const char *s, int len);
-    static QRgb convertQRgb(const ColourAllocated &col, unsigned alpha);
-    static QColor convertQColor(const ColourAllocated &col,
-            unsigned alpha = 0xff);
+    static QColor convertQColor(const ColourDesired &col,
+            unsigned alpha = 255);
+    static QRgb convertQRgb(const ColourDesired &col, unsigned alpha);
 
     bool unicodeMode;
     QPaintDevice *pd;
@@ -223,7 +242,7 @@ private:
     int pen_x, pen_y;
 };
 
-Surface *Surface::Allocate()
+Surface *Surface::Allocate(int)
 {
     return new SurfaceImpl;
 }
@@ -267,11 +286,21 @@ void SurfaceImpl::Init(QPainter *p)
     painter = p;
 }
 
-void SurfaceImpl::InitPixMap(int width, int height, Surface *, WindowID)
+void SurfaceImpl::InitPixMap(int width, int height, Surface *, WindowID wid)
 {
     Release();
 
-    pd = new QPixmap(width, height);
+#if QT_VERSION >= 0x050100
+    int dpr = PWindow(wid)->devicePixelRatio();
+    QPixmap *pixmap = new QPixmap(width * dpr, height * dpr);
+    pixmap->setDevicePixelRatio(dpr);
+#else
+    QPixmap *pixmap = new QPixmap(width, height);
+    Q_UNUSED(wid);
+#endif
+
+    pd = pixmap;
+
     painter = new QPainter(pd);
     my_resources = true;
 }
@@ -311,15 +340,15 @@ void SurfaceImpl::LineTo(int x_, int y_)
     pen_y = y_;
 }
 
-void SurfaceImpl::PenColour(ColourAllocated fore)
+void SurfaceImpl::PenColour(ColourDesired fore)
 {
     Q_ASSERT(painter);
 
     painter->setPen(convertQColor(fore));
 }
 
-void SurfaceImpl::Polygon(Point *pts, int npts, ColourAllocated fore,
-        ColourAllocated back)
+void SurfaceImpl::Polygon(Point *pts, int npts, ColourDesired fore,
+        ColourDesired back)
 {
     Q_ASSERT(painter);
 
@@ -333,23 +362,23 @@ void SurfaceImpl::Polygon(Point *pts, int npts, ColourAllocated fore,
     painter->drawPolygon(qpts);
 }
 
-void SurfaceImpl::RectangleDraw(PRectangle rc, ColourAllocated fore,
-        ColourAllocated back)
+void SurfaceImpl::RectangleDraw(PRectangle rc, ColourDesired fore,
+        ColourDesired back)
 {
     Q_ASSERT(painter);
 
     painter->setPen(convertQColor(fore));
     painter->setBrush(convertQColor(back));
-    painter->drawRect(rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top);
+    drawRect(rc);
 }
 
-void SurfaceImpl::FillRectangle(PRectangle rc, ColourAllocated back)
+void SurfaceImpl::FillRectangle(PRectangle rc, ColourDesired back)
 {
     Q_ASSERT(painter);
 
     painter->setPen(Qt::NoPen);
     painter->setBrush(convertQColor(back));
-    painter->drawRect(rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top);
+    drawRect(rc);
 }
 
 void SurfaceImpl::FillRectangle(PRectangle rc, Surface &surfacePattern)
@@ -365,15 +394,16 @@ void SurfaceImpl::FillRectangle(PRectangle rc, Surface &surfacePattern)
 
         painter->setPen(Qt::NoPen);
         painter->setBrush(brsh);
-        painter->drawRect(rc.left, rc.top, rc.right - rc.left,
-                rc.bottom - rc.top);
+        drawRect(rc);
     }
     else
-        FillRectangle(rc, ColourAllocated(0));
+    {
+        FillRectangle(rc, ColourDesired(0));
+    }
 }
 
-void SurfaceImpl::RoundedRectangle(PRectangle rc, ColourAllocated fore,
-        ColourAllocated back)
+void SurfaceImpl::RoundedRectangle(PRectangle rc, ColourDesired fore,
+        ColourDesired back)
 {
     Q_ASSERT(painter);
 
@@ -384,20 +414,20 @@ void SurfaceImpl::RoundedRectangle(PRectangle rc, ColourAllocated fore,
 }
 
 void SurfaceImpl::AlphaRectangle(PRectangle rc, int cornerSize,
-        ColourAllocated fill, int alphaFill, ColourAllocated outline,
+        ColourDesired fill, int alphaFill, ColourDesired outline,
         int alphaOutline, int)
 {
     Q_ASSERT(painter);
 
     int w = rc.right - rc.left;
     int h = rc.bottom - rc.top;
-
     QImage img(w, h, 32);
 
     img.fill(convertQRgb(fill, alphaFill));
     img.setAlphaBuffer(true);
 
-    // Assume that "cornerSize" means outline width.
+    // Assume that "cornerSize" means outline width (which it doesn't but keep
+    // this for historical reasons).
     if (cornerSize > 0)
     {
         QRgb oline = convertQRgb(outline, alphaOutline);
@@ -428,8 +458,13 @@ void SurfaceImpl::AlphaRectangle(PRectangle rc, int cornerSize,
     painter->drawImage(rc.left, rc.top, img);
 }
 
-void SurfaceImpl::Ellipse(PRectangle rc, ColourAllocated fore,
-        ColourAllocated back)
+void SurfaceImpl::drawRect(const PRectangle &rc)
+{
+    painter->drawRect(rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top);
+}
+
+void SurfaceImpl::Ellipse(PRectangle rc, ColourDesired fore,
+        ColourDesired back)
 {
     Q_ASSERT(painter);
 
@@ -458,17 +493,17 @@ void SurfaceImpl::Copy(PRectangle rc, Point from, Surface &surfaceSource)
     }
 }
 
-void SurfaceImpl::DrawTextNoClip(PRectangle rc, Font &font_, int ybase,
-        const char *s, int len,ColourAllocated fore, ColourAllocated back)
+void SurfaceImpl::DrawTextNoClip(PRectangle rc, Font &font_, XYPOSITION ybase,
+        const char *s, int len, ColourDesired fore, ColourDesired back)
 {
     Q_ASSERT(painter);
 
     FillRectangle(rc, back);
-    DrawTextTransparent(rc, font_, ybase, s, len, fore);
+    drawText(rc, font_, ybase, s, len, fore);
 }
 
-void SurfaceImpl::DrawTextClipped(PRectangle rc, Font &font_, int ybase,
-        const char *s, int len, ColourAllocated fore, ColourAllocated back)
+void SurfaceImpl::DrawTextClipped(PRectangle rc, Font &font_, XYPOSITION ybase,
+        const char *s, int len, ColourDesired fore, ColourDesired back)
 {
     Q_ASSERT(painter);
 
@@ -477,11 +512,21 @@ void SurfaceImpl::DrawTextClipped(PRectangle rc, Font &font_, int ybase,
     painter->setClipping(false);
 }
 
-void SurfaceImpl::DrawTextTransparent(PRectangle rc, Font &font_, int ybase,
-        const char *s, int len, ColourAllocated fore)
+void SurfaceImpl::DrawTextTransparent(PRectangle rc, Font &font_,
+        XYPOSITION ybase, const char *s, int len, ColourDesired fore)
 {
-    Q_ASSERT(painter);
+    // Only draw if there is a non-space.
+    for (int i = 0; i < len; ++i)
+        if (s[i] != ' ')
+        {
+            drawText(rc, font_, ybase, s, len, fore);
+            return;
+        }
+}
 
+void SurfaceImpl::drawText(const PRectangle &rc, Font &font_, XYPOSITION ybase,
+        const char *s, int len, ColourDesired fore)
+{
     QString qs = convertText(s, len);
 
     QFont *f = PFont(font_.GetID());
@@ -497,51 +542,73 @@ void SurfaceImpl::DrawXPM(PRectangle rc, const XPM *xpm)
 {
     Q_ASSERT(painter);
 
-    int x, y;
+    XYPOSITION x, y;
     const QPixmap &qpm = xpm->Pixmap();
 
-    x = rc.left + (rc.Width() - qpm.width()) / 2;
-    y = rc.top + (rc.Height() - qpm.height()) / 2;
+    x = rc.left + (rc.Width() - qpm.width()) / 2.0;
+    y = rc.top + (rc.Height() - qpm.height()) / 2.0;
 
     painter->drawPixmap(x, y, qpm);
 }
 
+void SurfaceImpl::DrawRGBAImage(PRectangle rc, int width, int height,
+        const unsigned char *pixelsImage)
+{
+    Q_ASSERT(painter);
+
+    const QImage *qim = reinterpret_cast<const QImage *>(pixelsImage);
+
+    painter->drawImage(rc.left, rc.top, *qim);
+}
+
 void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len,
-        int *positions)
+        XYPOSITION *positions)
 {
-    QFontMetrics fm = metrics(font_);
     QString qs = convertText(s, len);
-    int totalWidth = 0, ui = 0;
+    QFontMetrics fm = metrics(font_);
 
-    for (int i = 0; i < qs.length(); ++i)
+    // The position for each byte of a character is the offset from the start
+    // where the following character should be drawn.
+    int i_byte = 0;
+
+    for (int i_char = 0; i_char < qs.length(); ++i_char)
     {
-        totalWidth += fm.width(qs[i]);
+        // We can't just add the individual character widths together because
+        // of kerning.
+        XYPOSITION width = qRound(fm.width(qs.left(i_char + 1)));
 
-        int l = (unicodeMode ? QString(qs[i]).utf8().length() : 1);
+        if (unicodeMode)
+        {
+            // Set the same position for each byte of the character.
+            int nbytes = qs.mid(i_char, 1).utf8().length();
 
-        while (l--)
-            positions[ui++] = totalWidth;
+            while (nbytes--)
+                positions[i_byte++] = width;
+        }
+        else
+        {
+            positions[i_byte++] = width;
+        }
     }
 }
 
-int SurfaceImpl::WidthText(Font &font_, const char *s, int len)
+XYPOSITION SurfaceImpl::WidthText(Font &font_, const char *s, int len)
 {
-    QString qs = convertText(s, len);
+    return metrics(font_).width(convertText(s, len));
 
-    return metrics(font_).width(qs, qs.length());
 }
 
-int SurfaceImpl::WidthChar(Font &font_, char ch)
+XYPOSITION SurfaceImpl::WidthChar(Font &font_, char ch)
 {
     return metrics(font_).width(ch);
 }
 
-int SurfaceImpl::Ascent(Font &font_)
+XYPOSITION SurfaceImpl::Ascent(Font &font_)
 {
     return metrics(font_).ascent();
 }
 
-int SurfaceImpl::Descent(Font &font_)
+XYPOSITION SurfaceImpl::Descent(Font &font_)
 {
     // Qt doesn't include the baseline in the descent, so add it.  Note that
     // a descent from Qt4 always seems to be 2 pixels larger (irrespective of
@@ -551,18 +618,27 @@ int SurfaceImpl::Descent(Font &font_)
     return metrics(font_).descent() + 1;
 }
 
-int SurfaceImpl::ExternalLeading(Font &font_)
+XYPOSITION SurfaceImpl::ExternalLeading(Font &font_)
 {
     // Scintilla doesn't use this at the moment, which is good because Qt4 can
     // return a negative value.
     return metrics(font_).leading();
 }
 
-int SurfaceImpl::Height(Font &font_)
+XYPOSITION SurfaceImpl::Height(Font &font_)
 {
     return metrics(font_).height();
 }
 
+XYPOSITION SurfaceImpl::AverageCharWidth(Font &font_)
+{
+#if QT_VERSION >= 0x040200
+    return metrics(font_).averageCharWidth();
+#else
+    return WidthChar(font_, 'n');
+#endif
+}
+
 void SurfaceImpl::SetClip(PRectangle rc)
 {
     Q_ASSERT(painter);
@@ -575,15 +651,21 @@ void SurfaceImpl::FlushCachedState()
 {
 }
 
+// Return the QFont for a Font.
+QFont SurfaceImpl::convertQFont(Font &font)
+{
+    QFont *f = PFont(font.GetID());
+
+    if (f)
+        return *f;
+
+    return QApplication::font();
+}
+
 // Get the metrics for a font.
 QFontMetrics SurfaceImpl::metrics(Font &font_)
 {
-    QFont *f = PFont(font_.GetID()), fnt;
-
-    if (f)
-        fnt = *f;
-    else
-        fnt = QApplication::font();
+    QFont fnt = convertQFont(font_);
 
     if (painter && painter->isActive())
     {
@@ -605,7 +687,7 @@ QString SurfaceImpl::convertText(const char *s, int len)
 }
 
 // Convert a Scintilla colour and alpha component to a Qt QRgb.
-QRgb SurfaceImpl::convertQRgb(const ColourAllocated &col, unsigned alpha)
+QRgb SurfaceImpl::convertQRgb(const ColourDesired &col, unsigned alpha)
 {
     long c = col.AsLong();
 
@@ -613,15 +695,16 @@ QRgb SurfaceImpl::convertQRgb(const ColourAllocated &col, unsigned alpha)
     unsigned g = (c >> 8) & 0xff;
     unsigned b = (c >> 16) & 0xff;
 
-    QRgb rgba = (alpha << 24) | (r << 16) | (g << 8) | b;
-
-    return rgba;
+    return qRgba(r, g, b, alpha);
 }
 
-// Convert a Scintilla colour, and optional alpha component, to a Qt QColor.
-QColor SurfaceImpl::convertQColor(const ColourAllocated &col, unsigned alpha)
+// Convert a Scintilla colour, and alpha component, to a Qt QColor.
+QColor SurfaceImpl::convertQColor(const ColourDesired &col, unsigned alpha)
 {
-    return QColor(convertQRgb(col, alpha));
+    QColor c;
+    c.setRgb(convertQRgb(col, alpha));
+
+    return c;
 }
 
 
@@ -632,7 +715,7 @@ Window::~Window()
 
 void Window::Destroy()
 {
-    QWidget *w = PWindow(id);
+    QWidget *w = PWindow(wid);
 
     if (w)
     {
@@ -643,18 +726,18 @@ void Window::Destroy()
         // around the problem but this is the simplest and doesn't seem to
         // cause problems of its own.
         w->deleteLater();
-        id = 0;
+        wid = 0;
     }
 }
 
 bool Window::HasFocus()
 {
-    return PWindow(id)->hasFocus();
+    return PWindow(wid)->hasFocus();
 }
 
 PRectangle Window::GetPosition()
 {
-    QWidget *w = PWindow(id);
+    QWidget *w = PWindow(wid);
 
     // Before any size allocated pretend its big enough not to be scrolled.
     PRectangle rc(0,0,5000,5000);
@@ -672,19 +755,19 @@ PRectangle Window::GetPosition()
 
 void Window::SetPosition(PRectangle rc)
 {
-    PWindow(id)->setGeometry(rc.left, rc.top, rc.right - rc.left,
+    PWindow(wid)->setGeometry(rc.left, rc.top, rc.right - rc.left,
             rc.bottom - rc.top);
 }
 
 void Window::SetPositionRelative(PRectangle rc, Window relativeTo)
 {
-    QWidget *rel = PWindow(relativeTo.id);
+    QWidget *rel = PWindow(relativeTo.wid);
     QPoint pos = rel->mapToGlobal(rel->pos());
 
     int x = pos.x() + rc.left;
     int y = pos.y() + rc.top;
 
-    PWindow(id)->setGeometry(x, y, rc.right - rc.left, rc.bottom - rc.top);
+    PWindow(wid)->setGeometry(x, y, rc.right - rc.left, rc.bottom - rc.top);
 }
 
 PRectangle Window::GetClientPosition()
@@ -694,7 +777,7 @@ PRectangle Window::GetClientPosition()
 
 void Window::Show(bool show)
 {
-    QWidget *w = PWindow(id);
+    QWidget *w = PWindow(wid);
 
     if (show)
         w->show();
@@ -704,7 +787,7 @@ void Window::Show(bool show)
 
 void Window::InvalidateAll()
 {
-    QWidget *w = PWindow(id);
+    QWidget *w = PWindow(wid);
 
     if (w)
         w->update();
@@ -712,7 +795,7 @@ void Window::InvalidateAll()
 
 void Window::InvalidateRectangle(PRectangle rc)
 {
-    QWidget *w = PWindow(id);
+    QWidget *w = PWindow(wid);
 
     if (w)
         w->update(rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top);
@@ -720,7 +803,7 @@ void Window::InvalidateRectangle(PRectangle rc)
 
 void Window::SetFont(Font &font)
 {
-    PWindow(id)->setFont(*PFont(font.GetID()));
+    PWindow(wid)->setFont(*PFont(font.GetID()));
 }
 
 void Window::SetCursor(Cursor curs)
@@ -754,15 +837,17 @@ void Window::SetCursor(Cursor curs)
         break;
 
     default:
+        // Note that Qt doesn't have a standard cursor that could be used to
+        // implement cursorReverseArrow.
         qc = Qt::ArrowCursor;
     }
 
-    PWindow(id)->setCursor(qc);
+    PWindow(wid)->setCursor(qc);
 }
 
 void Window::SetTitle(const char *s)
 {
-    PWindow(id)->setCaption(s);
+    PWindow(wid)->setCaption(s);
 }
 
 
@@ -773,30 +858,30 @@ PRectangle Window::GetMonitorRect(Point)
 
 
 // Menu management.
-Menu::Menu() : id(0)
+Menu::Menu() : mid(0)
 {
 }
 
 void Menu::CreatePopUp()
 {
     Destroy();
-    id = new SciPopup();
+    mid = new QsciSciPopup();
 }
 
 void Menu::Destroy()
 {
-    SciPopup *m = PMenu(id);
+    QsciSciPopup *m = PMenu(mid);
 
     if (m)
     {
         delete m;
-        id = 0;
+        mid = 0;
     }
 }
 
 void Menu::Show(Point pt, Window &)
 {
-    PMenu(id)->popup(QPoint(pt.x, pt.y));
+    PMenu(mid)->popup(QPoint(pt.x, pt.y));
 }
 
 
@@ -818,7 +903,7 @@ public:
     virtual Function FindFunction(const char *name)
     {
         if (m)
-            return m->resolve(name);
+            return (Function)m->resolve(name);
 
         return 0;
     }
@@ -1003,3 +1088,5 @@ void Platform::Assert(const char *c, const char *file, int line)
 {
     qFatal("Assertion [%s] failed at %s %d\n", c, file, line);
 }
+
+QSCI_END_SCI_NAMESPACE
